#include <math.h>

void AIDEAL(int LPHI, int LPHID, int LPHIT, int LPHIDD, int LPHITT, int LPHIDT,
            int LPHDDD, int LPHDDT, int LPHDTT, double *PHI, double *PHID, double *PHIT,
            double *PHIDD, double *PHITT, double *PHIDT, double *PHIDDD, double *PHIDDT, 
            double *PHIDTT, double TAU, double DEL)
{
/*
NOTE: all of the Fortran LOGICAL variables (eg, LPHI) have been converted to int's in this 
      C translation.  In C, FALSE is simply 0, and TRUE is non-zero.

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** AIDEAL ***                                   C
C THIS ROUTINE COMPUTES THE REDUCED IDEAL-GAS HELMHOLTZ ENERGY AND ITS C
C (REDUCED) TEMPERATURE AND DENSITY DERIVATIVES FROM THE PRUSS AND     C
C WAGNER EOS FOR WATER AND STEAM.  ONLY THE QUANTITIES REQUESTED VIA   C
C THE LOGICAL ARGUMENTS ARE RETURNED.                                  C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C LXXXX   L    I   FOR WHETHER OR NOT TO COMPUTE QUANTITY XXXX         C
C PHI     R    O   REDUCED IDEAL-GAS HELMHOLTZ ENERGY                  C
C PHID    R    O   D(PHI)/D(DEL)                                       C
C PHIT    R    O   D(PHI)/D(TAU)                                       C
C PHIDD   R    O   D2(PHI)/D(DEL2)                                     C
C PHITT   R    O   D2(PHI)/D(TAU2)                                     C
C PHIDT   R    O   D2(PHI)/D(DEL)D(TAU)                                C
C PHIDDD  R    O   D3(PHI)/D(DEL3)                                     C
C PHIDDT  R    O   D3(PHI)/D(DEL2)D(TAU)                               C
C PHIDTT  R    O   D3(PHI)/D(DEL)D(TAU2)                               C
C TAU     R    I   DIMENSIONLESS INVERSE TEMPERATURE TC/T              C
C DEL     R    I   DIMENSIONLESS DENSITY RHO/RHOC                      C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 07JUL95 - INITIAL CREATION BY AHH                                    C
C 18AUG95 - MODIFIED VERSION AFTER DEBUGGING BY AHH AND JW             C
C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
C 16JAN97 - AHH: ADD THIRD DERIVATIVES                                 C
C 11FEB97 - AHH: MAKE EXP'S UNIFORMLY DOUBLE PRECISION                 C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/

/* 
C (MUST INSERT COMMON CONTAINING WAGNER COEFFICIENTS)
      INCLUDE 'COEFIG.CMN'
      
NOTE: Fortran used a common block and a block data routine to set these constants, for this 'C'
      translation, I have simply initialized them here.  For the WGAM0 array, only the indices
      4 through 8 are used.  As always, the C arrays are longer than the original Fortan 
      arrays by 1 as they start at the index [0].  Values that are not initialized are
      defaulted to '0.0'.
*/
	static const double WA0[] = { 0.0, -8.32044648375, 6.683210527593, 3.00632, 0.012436,
                                  0.97315, 1.27950, 0.96956, 0.24873 };

    static const double WGAM0[9] = { [4] = 1.28728967, [5] = 3.53734222, [6] = 7.74073708,
                                     [7] = 9.24437796, [8] = 27.5075105 };

	double EGAM, DIFF;
/*
C  INITIALIZE OUTPUT QUANTITIES TO ZERO
*/
    *PHI = 0.0;
    *PHID = 0.0;
    *PHIDD = 0.0;
    *PHIT = 0.0;
    *PHITT = 0.0;
    *PHIDT = 0.0;
    *PHIDDD = 0.0;
    *PHIDDT = 0.0;
    *PHIDTT = 0.0;
/*
C  START WITH THE TERMS NOT IN THE SUM
*/
    if (LPHI) 
    	*PHI = log(DEL) + WA0[1] + WA0[2]*TAU + WA0[3]*log(TAU);
      
    if (LPHID) 
    	*PHID = 1.0/DEL;
      
    if (LPHIDD) 
        *PHIDD = -1.0/(DEL*DEL);
      
    if (LPHIT) 
        *PHIT = WA0[2] + WA0[3]/TAU;
      
    if (LPHITT) 
        *PHITT = -WA0[3]/(TAU*TAU);
      
    if (LPHDDD) 
        *PHIDDD = 2.0 / pow(DEL, 3);

/*
C NOTE NO PHIDT MODIFICATION BECAUSE THAT IS IDENTICALLY ZERO

C ADD IN TERMS FROM THE SUMMATION FOR PHI AND THE T DERIVATIVES
C AS NEEDED
*/
    if (LPHI || LPHIT || LPHITT) {
      
    	for (int i = 4; i<9; i++) {
    	
          	EGAM = exp(-WGAM0[i]*TAU);
          	DIFF = 1.0 - EGAM;
          
          	if (LPHI) 
            	*PHI = *PHI + WA0[i]*log(DIFF);
          
          	if (LPHIT) 
            	*PHIT = *PHIT + WA0[i]*WGAM0[i]*(1.0/DIFF - 1.0);
          
          	if (LPHITT) 
            	*PHITT = *PHITT - WA0[i]*WGAM0[i]*WGAM0[i]*EGAM/(DIFF*DIFF);
 		}
 	}
      
    return;
}


void ARESID(int LPHI, int LPHID, int LPHIT, int LPHIDD, int LPHITT, int LPHIDT,
            int LPHDDD, int LPHDDT, int LPHDTT, double *PHI, double *PHID, double *PHIT,
            double *PHIDD, double *PHITT, double *PHIDT, double *PHIDDD, double *PHIDDT, 
            double *PHIDTT, double TAU, double DEL)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** ARESID ***                                   C
C THIS ROUTINE COMPUTES THE REDUCED RESIDUAL HELMHOLTZ ENERGY AND ITS  C
C (REDUCED) TEMPERATURE AND DENSITY DERIVATIVES FROM THE PRUSS AND     C
C WAGNER EOS FOR WATER AND STEAM.  ONLY THE QUANTITIES REQUESTED VIA   C
C THE LOGICAL ARGUMENTS ARE RETURNED.                                  C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C LXXXX   L    I   FOR WHETHER OR NOT TO COMPUTE QUANTITY XXXX         C
C PHI     R    O   REDUCED RESIDUAL HELMHOLTZ ENERGY                   C
C PHID    R    O   D(PHI)/D(DEL)                                       C
C PHIT    R    O   D(PHI)/D(TAU)                                       C
C PHIDD   R    O   D2(PHI)/D(DEL2)                                     C
C PHITT   R    O   D2(PHI)/D(TAU2)                                     C
C PHIDT   R    O   D2(PHI)/D(DEL)D(TAU)                                C
C PHIDDD  R    O   D3(PHI)/D(DEL3)                                     C
C PHIDDT  R    O   D3(PHI)/D(DEL2)D(TAU)                               C
C PHIDTT  R    O   D3(PHI)/D(DEL)D(TAU2)                               C
C TAU     R    I   DIMENSIONLESS RECIPROCAL TEMPERATURE TC/T           C
C DEL     R    I   DIMENSIONLESS DENSITY RHO/RHOC                      C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 06JUL95 - INITIAL CREATION BY AHH                                    C
C 18AUG95 - MODIFIED VERSION AFTER DEBUGGING BY AHH AND JW             C
C 19OCT95 - AHH: SPEED ENHANCEMENTS                                    C
C 21JUN96 - AHH: CORRECT DOCUMENTATION OF TAU                          C
C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
C 25SEP96 - AHH: AVOID UNDERFLOWS                                      C
C 16JAN97 - AHH: ADD THIRD DERIVATIVES                                 C
C 11FEB97 - AHH: MAKE EXP'S UNIFORMLY DOUBLE PRECISION                 C
C 21MAR97 - AHH: CORRECT ERROR IN PHIDDD                               C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/

/*
C
C (MUST INSERT COMMON CONTAINING WAGNER COEFFICIENTS)
      INCLUDE 'coef.cmn'
      
NOTE: in this C translation, I've just simply intialized them below.  As always, the C arrays have 
      been padded with a zero in the [0] location so that the indices will match those of the
      original Fortran arrays.  Also, as all C arrays start at [0], extra zeroes have been added 
      for the "missing" elements in the original Fortran arrays that were not allocated 
      (eg, IWC(8:51), and IWALF(52:54)).
*/      
	static const int IWC[] = { 0, 0, 0, 0, 0, 0, 0, 0,
	                           1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	                           2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
	                           2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 6, 6, 6, 6 };    
    

    static const int  IWD[] = { 0, 1, 1, 1, 2, 2, 3, 4, 1, 1, 1, 2, 2, 3, 4, 4, 5, 7, 9,
                               10, 11, 13, 15, 1, 2, 2, 2, 3, 4, 4, 4, 5, 6, 6, 7, 9, 9,
                                9, 9, 9, 10, 10, 12, 3, 4, 4, 5,14, 3, 6, 6, 6, 3, 3, 3 };
     
    static const double WT[] = { 0.0, -0.5, 0.875, 1.0, 0.5, 0.75, 0.375, 1.0 };
      
    static const int IWT[] = { 0, 0, 0, 0, 0, 0, 0, 0,
                               4, 6, 12, 1, 5, 4, 2, 13, 9, 3, 4, 11, 4, 13, 1, 7, 1, 9,
                              10, 10, 3, 7, 10, 10, 6, 10, 10, 1, 2, 3, 4, 8, 6, 9, 8, 16,
                              22, 23, 23, 10, 50, 44, 46, 50, 0, 1, 4 };
     
    static const double WN[] = { 0.0, 1.2533547935523e-02,  7.8957634722828,
                                     -8.7803203303561,      3.1802509345418e-01,
                                     -2.6145533859358e-01, -7.8199751687981e-03,
                                      8.8089493102134e-03, -6.6856572307965e-01,
                                      2.0433810950965e-01, -6.6212605039687e-05,
                                     -1.9232721156002e-01, -2.5709043003438e-01,
                                      1.6074868486251e-01, -4.0092828925807e-02,
                                      3.9343422603254e-07, -7.5941377088144e-06,
                                      5.6250979351888e-04, -1.5608652257135e-05,
                                      1.1537996422951e-09,  3.6582165144204e-07,
                                     -1.3251180074668e-12, -6.2639586912454e-10,
                                     -1.0793600908932e-01,  1.7611491008752e-02,
                                      2.2132295167546e-01, -4.0247669763528e-01,
                                      5.8083399985759e-01,  4.9969146990806e-03,
                                     -3.1358700712549e-02, -7.4315929710341e-01,
                                      4.7807329915480e-01,  2.0527940895948e-02,
                                     -1.3636435110343e-01,  1.4180634400617e-02,
                                      8.3326504880713e-03, -2.9052336009585e-02,
                                      3.8615085574206e-02, -2.0393486513704e-02,
                                     -1.6554050063734e-03,  1.9955571979541e-03,
                                      1.5870308324157e-04, -1.6388568342530e-05,
                                      4.3613615723811e-02,  3.4994005463765e-02,
                                     -7.6788197844621e-02,  2.2446277332006e-02,
                                     -6.2689710414685e-05, -5.5711118565645e-10,
                                     -1.9905718354408e-01,  3.1777497330738e-01,
                                     -1.1841182425981e-01, -3.1306260323435e+01,
                                      3.1546140237781e+01, -2.5213154341695e+03,
                                     -1.4874640856724e-01,  3.1806110878444e-01 };
     
    static const int IWALF[55] = { [52] = 20, [53] = 20, [54] = 20 };
    
    static const int IWBET[55] = { [52] = 150, [53] = 150, [54] = 250 };
    
    static const double WGAM[55] = { [52] = 1.21, [53] = 1.21, [54] = 1.25 };
    
    static const int IWEPS[55] = { [52] = 1, [53] = 1, [54] = 1 };
    
    static const double WA[57] = { [55] = 3.5, [56] = 3.5 };
      
    static const double WB[57] = { [55] = 0.85, [56] = 0.95 };
      
    static const double WBB[57] = { [55] = 0.2, [56] = 0.2 };
      
    static const int IWCC[57] = { [55] = 28, [56] = 32 };
      
    static const int IWDD[57] = { [55] = 700, [56] = 800 };
      
    static const double WAA[57] = { [55] = 0.32, [56] = 0.32 };
      
    static const double WBET[57] = { [55] = 0.3, [56] = 0.3 };

	int IC2, IC3, IS1, IS2, IS3, IS4, IA2E, IAE2; 
	double DINV, DINV2, DINV3, TINV, TINV2, DELPOW, TAUPOW, WNDT, DELC, WNDTE, PROD1;
	double DEPS, TGAM, ALFEXP, ALFBET, BETEXP, Q, DM1, TM1, PEXP, PSI, THETA, DELTA;
	double BM1, BM2, BM3, AM1, AM2, AM3, DB, DBM1, DBM2, DBM3, DDDT, DDBDT, DPDT;
	double DQDD, OVERB, OBM1, DTHDD, DDDD, DDBDD, DPDD, D2DBTD, D2PDTD, OBM2;
	double D2THDD, D2DDD, D2DBDD, D2PDD, D2DBDT, D2PDT, D3THDD, D3DDD, D3DBDD, D3PDD;
	double D3DBDT, D3PDDT, D3DBTD, D3PDTT, EPOW, S2;
  
/*
C  INITIALIZE OUTPUT QUANTITIES TO ZERO
*/
    *PHI = 0.0;
    *PHID = 0.0;
    *PHIDD = 0.0;
    *PHIT = 0.0;
    *PHITT = 0.0;
    *PHIDT = 0.0;
    *PHIDDD = 0.0;
    *PHIDDT = 0.0;
    *PHIDTT = 0.0;

    DINV = 1.0 / DEL;
    DINV2 = DINV*DINV;
    DINV3 = DINV2*DINV;
    TINV = 1.0 / TAU;
    TINV2 = TINV*TINV;
/*
  COMPUTE THE FIRST SEVEN TERMS IN THE SUM FOR ANY OF THE RESIDUAL
  PROPERTY DERIVATIVES THAT ARE NEEDED
*/
    for (int i=1; i<8; i++) {
    
    	DELPOW = pow(DEL, IWD[i]);
        TAUPOW = pow(TAU, WT[i]);
        WNDT = WN[i]*DELPOW*TAUPOW; 
        
        if (LPHI) 
          *PHI = *PHI + WNDT;
               
        if (LPHID) 
          *PHID = *PHID + WNDT*IWD[i]*DINV;
                
        if (LPHIDD) 
          *PHIDD = *PHIDD + WNDT*IWD[i]*(IWD[i]-1)*DINV2;
                
        if (LPHIT) 
          *PHIT = *PHIT + WNDT*WT[i]*TINV;
                
        if (LPHITT) 
          *PHITT = *PHITT + WNDT*WT[i]*(WT[i]-1.0)*TINV2;
                
        if (LPHIDT) 
          *PHIDT = *PHIDT + WNDT*IWD[i]*DINV*WT[i]*TINV;
                
        if (LPHDDD) 
          *PHIDDD = *PHIDDD + WNDT*IWD[i]*(IWD[i]-1)*(IWD[i]-2)*DINV3;        
        
        if (LPHDDT) 
          *PHIDDT = *PHIDDT + WNDT*IWD[i]*(IWD[i]-1)*WT[i]*TINV*DINV2;        
        
        if (LPHDTT) 
          *PHIDTT = *PHIDTT + WNDT*IWD[i]*WT[i]*(WT[i]-1.0)*TINV2*DINV;        
	}
 
/*
  NOW DO THE SAME THING FOR TERMS 8 THROUGH 51
*/
    for (int i=8; i<52; i++) {
      
    	DELPOW = pow(DEL, IWD[i]);
        TAUPOW = pow(TAU, IWT[i]);
        DELC = pow(DEL, IWC[i]); 
        
        if (DELC > 700.) 
          	EPOW = 0.0;
        else
          	EPOW = exp(-DELC);
        
        WNDTE = WN[i]*DELPOW*TAUPOW*EPOW;
        
        if (LPHI) 
        	*PHI = *PHI + WNDTE;
        
        if (LPHID) 
          	*PHID = *PHID + WNDTE*DINV*(IWD[i]-IWC[i]*DELC);
        
        if (LPHIDD) {
          	PROD1 = IWC[i]*DELC;
          	*PHIDD = *PHIDD + WNDTE*DINV2*((IWD[i]-PROD1)*(IWD[i]-1.0-PROD1) - IWC[i]*PROD1);
        }
        
        if (LPHIT) 
          	*PHIT = *PHIT + WNDTE*IWT[i]*TINV;
        
        if (LPHITT) 
          	*PHITT = *PHITT + WNDTE*IWT[i]*(IWT[i]-1)*TINV2;
        
        if (LPHIDT) 
          	*PHIDT = *PHIDT + WNDTE*DINV*IWT[i]*TINV*(IWD[i] - IWC[i]*DELC);
        
        if (LPHDDD) {
          	IC2 = IWC[i]*IWC[i];
          	IC3 = IC2*IWC[i];
          	IS1 = -2*IWC[i] + 3*IC2 - IC3 + 6*IWC[i]*IWD[i] - 3*IWD[i]*(IC2 + IWC[i]*IWD[i]);
          	IS2 = 3 * (IC3 - IC2 + IC2*IWD[i]);
          	*PHIDDD = *PHIDDD + WNDTE*DINV3*(IWD[i]*(2-3*IWD[i]+IWD[i]*IWD[i]) + IS1*DELC
                    + IS2*DELC*DELC - IC3*pow(DELC, 3));
     	}
        
        if (LPHDDT) {
          	IC2 = IWC[i]*IWC[i];
          	IS1 = IWC[i] - IC2 - 2*IWC[i]*IWD[i];
          	*PHIDDT = *PHIDDT + WNDTE*IWT[i]*TINV*DINV2*(IWD[i]*IWD[i] - IWD[i] + IS1*DELC 
          	        + IC2*DELC*DELC);
        }
        
        if (LPHDTT) 
          	*PHIDTT = *PHIDTT + WNDTE*IWT[i]*(IWT[i]-1)*TINV2*DINV*(IWD[i] - IWC[i]*DELC);
	}
 
/*
   NOW DO THE SAME THING FOR TERMS 52 THROUGH 54
*/
    for (int i=52; i<55; i++) {

    	DELPOW = pow(DEL, IWD[i]);
        TAUPOW = pow(TAU, IWT[i]);
        DEPS = DEL - IWEPS[i];
        TGAM = TAU - WGAM[i];
        ALFEXP = IWALF[i]*DEPS*DEPS;
        BETEXP = IWBET[i]*TGAM*TGAM;
        ALFBET = ALFEXP + BETEXP;
        
        if (ALFBET > 700.0) 
          EPOW = 0.0;
        else
          EPOW = exp(-ALFBET);
        
        WNDTE = WN[i]*DELPOW*TAUPOW*EPOW;
        
        if (LPHI) 
        	*PHI = *PHI + WNDTE;
        
        if (LPHID) 
          	*PHID = *PHID + WNDTE*(IWD[i]*DINV - 2.0*IWALF[i]*DEPS);
        
        if (LPHIDD) 
          	*PHIDD = *PHIDD + WNDTE*(-2.0*IWALF[i] + 4.0*IWALF[i]*IWALF[i]*DEPS*DEPS 
                   - 4.0*IWD[i]*IWALF[i]*DINV*DEPS + IWD[i]*(IWD[i] - 1)*DINV2);
        
        if (LPHIT) 
          	*PHIT = *PHIT + WNDTE * (IWT[i]*TINV - 2.0*IWBET[i]*TGAM);
        
        if (LPHITT) 
          	*PHITT = *PHITT + WNDTE*(pow(IWT[i]*TINV - 2.0*IWBET[i]*TGAM, 2)
                   - IWT[i]*TINV2 - 2.0*IWBET[i]);
        
        if (LPHIDT) 
          	*PHIDT = *PHIDT + WNDTE*(IWD[i]*DINV - 2.0*IWALF[i]*DEPS)*(IWT[i]*TINV 
          	       - 2.0*IWBET[i]*TGAM);
        
        if (LPHDDD) {
          	IA2E = IWALF[i]*IWALF[i]*IWEPS[i];
          	IAE2 = IWALF[i]*IWEPS[i]*IWEPS[i];
          	IS1 = 6*IWALF[i]*IWD[i]*IWEPS[i]*(IWD[i] - 1);
          	IS2 = 6*IWALF[i]*IWD[i]*(2*IAE2 - IWD[i]);
          	IS3 = 4*IA2E*(2*IAE2 - 6*IWD[i] - 3);
          	IS4 = 12*IWALF[i]*IWALF[i]*(1 + IWD[i] - 2*IAE2);
          	
          	*PHIDDD = *PHIDDD + WNDTE*DINV3*(IWD[i]*(2 - 3*IWD[i] + IWD[i]*IWD[i]) 
          	        + DEL*(IS1 + DEL*(IS2 + DEL*(IS3 + DEL*(IS4 + DEL*(24*IWALF[i]*IA2E 
                    - 8*pow(IWALF[i],3)*DEL))))));
     	}
        
        if (LPHDDT) {
          	IS2 = 2*IWALF[i]*(2*IWALF[i]*IWEPS[i]*IWEPS[i] - 2*IWD[i] - 1);
          	
          	*PHIDDT = *PHIDDT + WNDTE*DINV2*TINV*(IWD[i]*(IWD[i] - 1) 
          	        + DEL*(4*IWALF[i]*IWD[i]*IWEPS[i] + DEL*(IS2 
          	        + DEL*IWALF[i]*IWALF[i]*(-8*IWEPS[i] + 4*DEL))))
          	        * (IWT[i] + 2*IWBET[i]*TAU*(WGAM[i] - TAU));
        }
        
        if (LPHDTT) {
          	S2 = 2*IWBET[i]*(2*IWBET[i]*WGAM[i]*WGAM[i] - 2*IWT[i] - 1);
          	
          	*PHIDTT = *PHIDTT + WNDTE*DINV*TINV2*(IWT[i]*(IWT[i] - 1) 
          	        + TAU*(4*IWBET[i]*WGAM[i]*IWT[i] + TAU*(S2 + TAU*(-8*IWBET[i]*IWBET[i]*WGAM[i]
          	        + 4*IWBET[i]*IWBET[i]*TAU))))*(IWD[i] + 2*IWALF[i]*DEL*(IWEPS[i] - DEL));
        }
	}

/*
   NOW FOR THE I=55 AND 56 TERMS
*/
    Q = pow(1.0 - DEL,2);
    DM1 = DEL - 1.0;
    TM1 = TAU - 1.0;
      
    for (int i=55; i<57; i++) {
    
    	PEXP = -IWCC[i]*Q - IWDD[i]*TM1*TM1;
    	
        if (PEXP < -700.0) 
          	PSI = 0.0;
        else
          	PSI = exp(PEXP);
        
        THETA = 1.0 - TAU + WAA[i]*pow(Q,0.50/WBET[i]);
        DELTA = THETA*THETA + WBB[i]*pow(Q,WA[i]);
        BM1 = WB[i] - 1.0;
        BM2 = BM1 - 1.0;
        BM3 = BM2 - 1.0;
        AM1 = WA[i] - 1.0;
        AM2 = AM1 - 1.0;
        AM3 = AM2 - 1.0;
        DB = pow(DELTA,WB[i]);
        
        if (DELTA == 0.0) {
          	DBM1 = 0.0;
          	DBM2 = 0.0;
          	DBM3 = 0.0;
        }
        else {
          	DBM1 = pow(DELTA,BM1);
          	DBM2 = pow(DELTA,BM2);
          	DBM3 = pow(DELTA,BM3);
        }
        
        if (LPHI) 
          	*PHI = *PHI + WN[i]*DB*DEL*PSI;
        
/*
   CERTAIN CALCULATIONS ARE NEEDED WHEN THERE IS AT LEAST ONE 
   DIFFERENTIATION WITH RESPECT TO TAU
*/
        if (LPHIT || LPHITT || LPHIDT || LPHDDT || LPHDTT) {
          	DDDT = -2.0*THETA;
          	DDBDT = WB[i]*DBM1*DDDT;
          	DPDT = -2.0*IWDD[i]*TM1*PSI;
        }
        
/*
   CERTAIN CALCULATIONS ARE NEEDED WHEN THERE IS AT LEAST ONE 
   DIFFERENTIATION WITH RESPECT TO DEL
*/
		if (LPHID || LPHIDD || LPHIDT || LPHDDD || LPHDDT || LPHDTT) {
          	DQDD = 2.0*DM1;
          	OVERB = 0.50/WBET[i];
          	OBM1 = OVERB - 1.0;
          	DTHDD = WAA[i]*OVERB * pow(Q,OBM1)*DQDD;
          	DDDD = 2.0*THETA*DTHDD + WBB[i]*WA[i]*pow(Q,AM1)*DQDD;
          	DDBDD = WB[i]*DBM1*DDDD;
          	DPDD = -2.0*IWCC[i]*DM1*PSI;
        }
        
/*
   CERTAIN CALCULATIONS ARE NEEDED WHEN THERE IS AT LEAST ONE 
   DIFFERENTIATION WITH RESPECT TO DEL AND ONE W.R.T. TAU
*/
        if (LPHIDT || LPHDDT || LPHDTT) {
          	D2DBTD = -2.0*WB[i]*(THETA*BM1*DBM2*DDDD + DTHDD*DBM1);
          	D2PDTD = 4.0*IWCC[i]*IWDD[i]*DM1*TM1*PSI;
        }
        
/*
   CERTAIN CALCULATIONS ARE NEEDED WHEN THERE ARE AT LEAST TWO 
   DIFFERENTIATIONS WITH RESPECT TO DEL
*/
        if (LPHIDD || LPHDDD || LPHDDT) {
        
          	OBM2 = OBM1 - 1.0;
          	
          	if (Q == 0.0) {
            	D2THDD = 0.0;
            	D2DDD = 2.0*(THETA*D2THDD + DTHDD*DTHDD);
            }
          	else {
           	 	D2THDD = WAA[i]*OVERB*(OBM1*pow(Q,OBM2)*DQDD*DQDD +   2.0*pow(Q,OBM1));
            	D2DDD = 2.0*(THETA*D2THDD + DTHDD*DTHDD) + WBB[i]*WA[i]*
                        (AM1*pow(Q,AM2)*DQDD*DQDD + 2.0*pow(Q,AM1));
        	}
        	
          	D2DBDD = WB[i]*(BM1*DBM2*DDDD*DDDD + DBM1*D2DDD);
          	D2PDD = 2.0*IWCC[i]*PSI*(2.0*IWCC[i]*Q - 1.0);
        }
        
/*
   CERTAIN CALCULATIONS ARE NEEDED WHEN THERE ARE AT LEAST TWO 
   DIFFERENTIATIONS WITH RESPECT TO TAU
*/
        if (LPHITT || LPHDTT) {
          	D2DBDT = 2.0*WB[i]*(2.0*THETA*THETA*BM1*DBM2 + DBM1);
          	D2PDT = 2.0*IWDD[i]*PSI*(2.0*IWDD[i]*TM1*TM1 - 1.0);
        }

        if (LPHID) 
          	*PHID = *PHID + WN[i]*(DB*(PSI+DEL*DPDD) + DDBDD*DEL*PSI);
        
        if (LPHIDD) 
          	*PHIDD = *PHIDD + WN[i]*(DB*(2.0*DPDD + DEL*D2PDD) 
                   + 2.0*DDBDD*(PSI + DEL*DPDD) + D2DBDD*DEL*PSI);
        
        if (LPHIT) 
          	*PHIT = *PHIT + WN[i]*DEL*(DDBDT*PSI + DB*DPDT);
        
        if (LPHITT) 
          	*PHITT = *PHITT + WN[i]*DEL*(D2DBDT*PSI + 2.0*DDBDT*DPDT + DB*D2PDT);
        
        if (LPHIDT) 
          	*PHIDT = *PHIDT + WN[i]*(DB*(DPDT + DEL*D2PDTD) + DEL*DDBDD*DPDT
                   + DDBDT*(PSI + DEL*DPDD) + D2DBTD*DEL*PSI);
        
        if (LPHDDD) {
        
          	if (Q == 0.0) {
            	D3THDD = 0.0;
            	D3DDD = 0.0;
            } 
            else {
            	D3THDD = WAA[i]*OVERB*OBM1 * (OBM2*pow(Q,OVERB-3.0)*pow(DQDD,3) 
            	       + 6.0*pow(Q,OBM2)*DQDD);
            	D3DDD = 6.0*DTHDD*D2THDD + 2.0*THETA*D3THDD + WBB[i]*WA[i]*AM1 
            	      * (AM2*pow(Q,AM3)*pow(DQDD,3) + 6.0*pow(Q,AM2)*DQDD);
          	}
          	
          D3DBDD = WB[i] * (BM1*(BM2*DBM3*pow(DDDD,3) + 3.0*DBM2*DDDD*D2DDD) + DBM1*D3DDD);
          D3PDD = 4*IWCC[i]*IWCC[i]*DM1*PSI * (3 - 2*IWCC[i]*DM1*DM1);
          
          *PHIDDD = *PHIDDD + WN[i]*(DB*(3.0*D2PDD + DEL*D3PDD) + 3.0*DDBDD*(2.0*DPDD + DEL*D2PDD)
                  + 3.0*D2DBDD*(PSI + DEL*DPDD) + D3DBDD*DEL*PSI);
        }
        
        if (LPHDDT) {
          	D3DBDT = -2.0*WB[i] * (BM1*(BM2*THETA*DBM3*DDDD*DDDD + 2.0*DBM2*DDDD*DTHDD 
          	       + THETA*DBM2*D2DDD) + DBM1*D2THDD);
           	D3PDDT = -4*IWCC[i]*IWDD[i]*TM1*PSI * (2*IWCC[i]*DM1*DM1 - 1.0);
          
          	*PHIDDT = *PHIDDT + WN[i]*(DB*(2.0*D2PDTD + DEL*D3PDDT) + DDBDT*(2.0*DPDD + DEL*D2PDD)
                    + 2.0*(DDBDD*(DPDT + DEL*D2PDTD) + D2DBTD*(PSI + DEL*DPDD))
                    + D2DBDD*DEL*DPDT + D3DBDT*DEL*PSI);
        }
        
        if (LPHDTT) {
          	D3DBTD = 2.0*WB[i]*BM1 * (4.0*THETA*DTHDD*DBM2 + 2.0*THETA*THETA*BM2*DBM3*DDDD
                   + DBM2*DDDD);
          	D3PDTT = -4*IWCC[i]*IWDD[i]*DM1*PSI * (2*IWDD[i]*TM1*TM1 -1.0);
          	
          	*PHIDTT = *PHIDTT + WN[i]*(DB*(D2PDT + DEL*D3PDTT) + 2.0*DDBDT*(DPDT + DEL*D2PDTD)
                    + DDBDD*DEL*D2PDT + D2DBDT*(PSI + DEL*DPDD)
                    + 2.0*D2DBTD*DEL*DPDT + D3DBTD*DEL*PSI);
        }
	}
	
    return;
}


/*
	NOTE: all of the block data initializations have been moved into the functions above that 
	      use the constants.  This comment block is retained for documentation purposes.
	      
	BLOCK DATA WDATA
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C THIS BLOCK DATA CONSTRUCT SETS THE COEFFECIENTS REQUIRED BY THE      C
C WATER EOS.  THE NAMING OF THE COEFFICIENTS FOLLOWS THE NOMENCLATURE  C
C IN THE PRUSS AND WAGNER PAPER TO THE EXTENT POSSIBLE.  INTEGER ARRAYSC
C OF COEFFICIENTS BEGIN WITH "IW" AND REALS WITH "W".  UPPER-CASE      C
C VARIABLES IN THE ORIGINAL PAPER ARE DONE BY DOUBLING THE LETTER. FOR C
C EXAMPLE, THE REALS GIVEN BY LOWER CASE B ARE WB, AND THE INTEGERS    C
C GIVEN BY UPPER CASE B ARE IWBB.  ONLY THOSE ARRAY ELEMENTS FOR WHICH C
C COEFFICIENTS ACTUALLY EXIST ARE ALLOCATED.  THEREFORE, SINCE THE     C
C INTEGERS GIVEN BY GREEK ALPHA ARE ONLY USED IN TERMS 52-54, IT IS    C
C ALLOCATED AS IWALF(52:54).                                           C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 23AUG96 - AS SUGGESTED IN RELEASE, ADJUST FIRST 2 NUMBERS IN IDEAL-  C
C           GAS EXPRESSION SLIGHTLY TO GET INTERNAL ENERGY AND ENTROPY C
C           ZEROS MORE PRECISE FOR LIQUID AT TRIPLE POINT.  THIS       C
C           PRODUCED TINY CHANGES IN WA0(1) AND WAO(2).  ALSO REMOVED  C
C           A VARIABLE THAT WAS DECLARED BUT NEVER USED.               C
C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
C 17SEP96 - AHH: BREAK UP WN INITIALIZATION SINCE IT IS NONSTANDARD    C
C                TO HAVE MORE THAN 19 CONTINUATION LINES               C
C 18SEP96 - AHH: REVISE WN BREAKUP SINCE MS COMPILER DIDN'T LIKE       C
C                YESTERDAY'S FIX.                                      C
C 24SEP96 - AHH: REMOVE IMPLICIT NONE (REDUNDANT AND NONSTANDARD F77)  C
C 21MAY97 - AHH: PARAMETERIZE WCONST.CMN, SO DON'T SET HERE            C
C 06AUG97 - AHH: CHANGE AN "E" TO "D" IN WN(1) TO PICK UP ALL THE      C
C                PRECISION.  THEN MUST READJUST WA0(1) AND WA0(2)      C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
